"""Authentication methods for local domain on win32"""

import sys
from string import lower
from zLOG import LOG, WARNING
import win32api
import win32net, win32netcon
import win32security
from unicode_support import make_string

def get_local_ntdom(encoding='iso-8859-15'):
    """Return the local NT Domain
    If the local machine is a domain controller, returns the current domain
    If the local machine is a standalone server, returns the machine name

    NT returns an unicode object. encode it in 'encoding'
    """
    infos = win32net.NetServerGetInfo(None, 101)
    type = infos['type']
    if (type & win32netcon.SV_TYPE_DOMAIN_CTRL) or (type & win32netcon.SV_TYPE_DOMAIN_BAKCTRL):
        local_ntdom = win32api.GetDomainName()
        # XXX - there *has* to be a way to get the domain
        # this computer is a controller for
    else:
        local_ntdom = infos['name']
    return lower(make_string(local_ntdom, encoding))

def get_usernames(ntdom=None, encoding='iso-8859-15'):
    """Return the users of the local NT Domain
    If ntdom is not None, usernames are prefixed with it

    Beware! returned usernames are not lowercase!

    NT returns a list of unicode objects. encode them in 'encoding'
    """
    resume = 0
    usernames = []

    # ask the domain controller
    while 1:
        users, junk, resume = win32net.NetUserEnum(None, 0, win32netcon.FILTER_NORMAL_ACCOUNT, resume)
        ntnames = map(lambda user, encoding=encoding: lower(make_string(user['name'], encoding)), users)
        if ntdom is not None:
            ntnames = map(lambda ntname, ntdom=ntdom: ntdom + '\\' + ntname, ntnames)
        usernames.extend(ntnames)
        if not resume:
            break
    return usernames

def get_userinfo(ntname, ntdom=None, encoding='iso-8859-15'):
    """Return the full name and the comment of a local user
    ntdom is ignored

    NT returns unicode objects for full_name and comment. encode them in
    'encoding'
    """
    try:
        userinfo = win32net.NetUserGetInfo(None, ntname, 20)
    except win32api.error:
        return None
    return {'full_name': make_string(userinfo['full_name'], encoding),
            'comment': make_string(userinfo['comment'], encoding),
            }

def authenticate_user(ntname, ntdom, password):
    token = None
    try:
        try:
            token = win32security.LogonUser(
                ntname,
                ntdom,
                password,
                win32security.LOGON32_LOGON_NETWORK,
                win32security.LOGON32_PROVIDER_DEFAULT)
            return 1
        except:
            LOG('NTUserFolder',
                WARNING,
                'Error trying to authenticate %s\%s' % (ntdom, ntname),
                'LogonUser raised an exception\n',
                sys.exc_info())
    finally:
        if token:
            win32api.CloseHandle(token)
    return 0

